==> pyecore_py2-0.7.2-py2-none-any.whl: pyecore/resources/json.py <==
"""
The json module introduces JSON resource and JSON parsing.
"""
from __future__ import absolute_import
import json
from .resource import Resource
from .. import ecore as Ecore


class JsonResource(Resource):
    def __init__(self, uri=None, use_uuid=False, indent=None):
        super(JsonResource, self).__init__(uri, use_uuid)
        self._resolve_later = []
        self._already_saved = []
        self._load_href = {}
        self.indent = indent

    def load(self):
        json_value = self.uri.create_instream()
        d = json.loads(json_value.read().decode('utf-8'))
        self.to_obj(d, first=True)
        self.uri.close_stream()
        for inst, refs in self._load_href.iteritems():
            self.process_inst(inst, refs)
        self._load_href.clear()

    def save(self, output=None):
        stream = self.open_out_stream(output)
        root = self.contents[0]  # Only single root atm
        with stream as out:
            out.write(json.dumps(self.to_dict(root), indent=self.indent)
                          .encode('utf-8'))

    def _uri_fragment(self, obj):
        if obj.eResource == self:
            use_id = self._use_uuid
        else:
            use_id = obj.eResource and obj.eResource._use_uuid
        if use_id:
            self._assign_uuid(obj)
            return obj._xmiid
        else:
            return obj.eURIFragment()

    def _to_ref_from_obj(self, obj):
        eclass = obj.eClass
        uri = '{}{}'.format(eclass.ePackage.nsURI,
                            obj.eClass.eURIFragment())
        ref = {'eClass': uri}
        if obj.eResource == self:
            resource_uri = ''
        else:
            resource_uri = obj.eResource.uri if obj.eResource else ''
        ref['$ref'] = '{}{}'.format(resource_uri, self._uri_fragment(obj))
        return ref

    def _to_dict_from_obj(self, obj):
        eclass = obj.eClass
        uri = '{}{}'.format(eclass.ePackage.nsURI, eclass.eURIFragment())
        d = {'eClass': uri}
        containingFeature = obj.eContainmentFeature()
        for attr in obj._isset:
            is_ereference = isinstance(attr, Ecore.EReference)
            is_ref = is_ereference and not attr.containment
            if is_ereference and attr.eOpposite:
                if attr.eOpposite is containingFeature:
                    continue
            value = obj.eGet(attr)
            if value == attr.get_default_value():
                continue
            d[attr.name] = self.to_dict(value, is_ref=is_ref)
            if self._use_uuid:
                self._assign_uuid(obj)
                d['uuid'] = obj._xmiid
        self._already_saved.append(obj)
        return d

    def to_dict(self, obj, is_ref=False):
        if isinstance(obj, Ecore.EObject):
            fun = self._to_ref_from_obj if is_ref else self._to_dict_from_obj
            return fun(obj)
        elif isinstance(obj, type) and issubclass(obj, Ecore.EObject):
            fun = self._to_ref_from_obj if is_ref else self._to_dict_from_obj
            return fun(obj.eClass)
        elif isinstance(obj, Ecore.ECollection):
            fun = self._to_ref_from_obj if is_ref else self.to_dict
            return [fun(x) for x in obj]
        else:
            return obj

    def to_obj(self, d, owning_feature=None, first=False):
        uri_eclass = d['eClass']
        is_ref = '$ref' in d
        if is_ref:
            return Ecore.EProxy(path=d['$ref'], resource=self)
        excludes = ['eClass', '$ref', 'uuid']
        decoders = self._get_href_decoder(uri_eclass)
        eclass = decoders.resolve(uri_eclass, self)
        if not eclass:
            raise ValueError('Unknown metaclass {} for uri {}'
                             .format(eclass, uri_eclass))
        if eclass in (Ecore.EClass.eClass, Ecore.EClass):
            inst = eclass(d['name'].encode())
            excludes.append('name')
        else:
            inst = eclass()
        if first:
            self._use_uuid = 'uuid' in d
            self.append(inst)

        if self._use_uuid:
            self.uuid_dict[d['uuid']] = inst

        eattributes = []
        containments = []
        ereferences = []
        eclass = inst.eClass
        for key, value in d.iteritems():
            if key in excludes:
                continue
            feature = eclass.findEStructuralFeature(key)
            if not feature:
                raise ValueError('Unknown feature {} for object "{}"'
                                 .format(key, eclass))
            if isinstance(feature, Ecore.EAttribute):
                eattributes.append((feature, value))
            elif isinstance(feature, Ecore.EReference):
                if feature.containment:
                    containments.append((feature, value))
                else:
                    ereferences.append((feature, value))
        self.process_inst(inst, eattributes)
        self.process_inst(inst, containments, owning_feature)
        self._load_href[inst] = ereferences
        return inst

    def process_inst(self, inst, features, owning_feature=None):
        for feature, value in features:
            if isinstance(value, dict):
                element = self.to_obj(value, owning_feature=feature)
                if feature.eOpposite is None or \
                        feature.eOpposite is not owning_feature:
                    inst.eSet(feature, element)
            elif isinstance(value, list):
                elements = [self.to_obj(x, owning_feature=feature)
                            for x in value]
                elements = [x for x in elements if x is not None]
                collection = inst.eGet(feature)
                if feature.eOpposite is None or \
                        feature.eOpposite is not owning_feature:
                    collection.extend(elements)
            elif isinstance(value, unicode):
                inst.eSet(feature, value.encode())
            elif not isinstance(value, str):
                inst.eSet(feature, feature.eType.from_string(value))
            else:
                inst.eSet(feature, value)

==> pyecore_py2-0.7.3-py2-none-any.whl: pyecore/resources/json.py <==
"""
The json module introduces JSON resource and JSON parsing.
"""
from __future__ import absolute_import
from backports.functools_lru_cache import lru_cache
import json
from .resource import Resource
from .. import ecore as Ecore


class JsonResource(Resource):
    def __init__(self, uri=None, use_uuid=False, indent=None):
        super(JsonResource, self).__init__(uri, use_uuid)
        self._resolve_later = []
        self._already_saved = []
        self._load_href = {}
        self.indent = indent

    def load(self):
        json_value = self.uri.create_instream()
        d = json.loads(json_value.read().decode('utf-8'))
        self.to_obj(d, first=True)
        self.uri.close_stream()
        for inst, refs in self._load_href.iteritems():
            self.process_inst(inst, refs)
        self._load_href.clear()

    def save(self, output=None):
        stream = self.open_out_stream(output)
        root = self.contents[0]  # Only single root atm
        stream.write(json.dumps(self.to_dict(root), indent=self.indent)
                     .encode('utf-8'))
        self.uri.close_stream()

    def _uri_fragment(self, obj):
        if obj.eResource == self:
            use_id = self._use_uuid
        else:
            use_id = obj.eResource and obj.eResource._use_uuid
        if use_id:
            self._assign_uuid(obj)
            return obj._xmiid
        else:
            return obj.eURIFragment()

    def _to_ref_from_obj(self, obj):
        eclass = obj.eClass
        uri = '{}{}'.format(eclass.eRoot().nsURI,
                            obj.eClass.eURIFragment())
        ref = {'eClass': uri}
        if obj.eResource == self:
            resource_uri = ''
        else:
            resource_uri = obj.eResource.uri if obj.eResource else ''
        ref['$ref'] = '{}{}'.format(resource_uri, self._uri_fragment(obj))
        return ref

    def _to_dict_from_obj(self, obj):
        eclass = obj.eClass
        uri = '{}{}'.format(eclass.eRoot().nsURI, eclass.eURIFragment())
        d = {'eClass': uri}
        containingFeature = obj.eContainmentFeature()
        for attr in obj._isset:
            is_ereference = isinstance(attr, Ecore.EReference)
            is_ref = is_ereference and not attr.containment
            if is_ereference and attr.eOpposite:
                if attr.eOpposite is containingFeature:
                    continue
            value = obj.eGet(attr)
            if value == attr.get_default_value():
                continue
            d[attr.name] = self.to_dict(value, is_ref=is_ref)
            if self._use_uuid:
                self._assign_uuid(obj)
                d['uuid'] = obj._xmiid
        self._already_saved.append(obj)
        return d

    def to_dict(self, obj, is_ref=False):
        if isinstance(obj, Ecore.EObject):
            fun = self._to_ref_from_obj if is_ref else self._to_dict_from_obj
            return fun(obj)
        elif isinstance(obj, type) and issubclass(obj, Ecore.EObject):
            fun = self._to_ref_from_obj if is_ref else self._to_dict_from_obj
            return fun(obj.eClass)
        elif isinstance(obj, Ecore.ECollection):
            fun = self._to_ref_from_obj if is_ref else self.to_dict
            return [fun(x) for x in obj]
        else:
            return obj

    @lru_cache()
    def resolve_eclass(self, uri_eclass):
        decoders = self._get_href_decoder(uri_eclass)
        return decoders.resolve(uri_eclass, self)

    def to_obj(self, d, owning_feature=None, first=False):
        uri_eclass = d['eClass']
        is_ref = '$ref' in d
        if is_ref:
            return Ecore.EProxy(path=d['$ref'], resource=self)
        excludes = ['eClass', '$ref', 'uuid']
        eclass = self.resolve_eclass(uri_eclass)
        if not eclass:
            raise ValueError('Unknown metaclass {} for uri {}'
                             .format(eclass, uri_eclass))
        if eclass in (Ecore.EClass.eClass, Ecore.EClass):
            inst = eclass(d['name'].encode())
            excludes.append('name')
        else:
            inst = eclass()
        if first:
            self._use_uuid = 'uuid' in d
            self.append(inst)

        if self._use_uuid:
            self.uuid_dict[d['uuid']] = inst

        eattributes = []
        containments = []
        ereferences = []
        eclass = inst.eClass
        for key, value in d.iteritems():
            if key in excludes:
                continue
            feature = eclass.findEStructuralFeature(key)
            if not feature:
                raise ValueError('Unknown feature {} for object "{}"'
                                 .format(key, eclass))
            if isinstance(feature, Ecore.EAttribute):
                eattributes.append((feature, value))
            elif isinstance(feature, Ecore.EReference):
                if feature.containment:
                    containments.append((feature, value))
                else:
                    ereferences.append((feature, value))
        self.process_inst(inst, eattributes)
        self.process_inst(inst, containments, owning_feature)
        self._load_href[inst] = ereferences
        return inst

    def process_inst(self, inst, features, owning_feature=None):
        for feature, value in features:
            if isinstance(value, dict):
                element = self.to_obj(value, owning_feature=feature)
                if feature.eOpposite is None or \
                        feature.eOpposite is not owning_feature:
                    inst.eSet(feature, element)
            elif isinstance(value, list):
                elements = [self.to_obj(x, owning_feature=feature)
                            for x in value]
                elements = [x for x in elements if x is not None]
                collection = inst.eGet(feature)
                if feature.eOpposite is None or \
                        feature.eOpposite is not owning_feature:
                    collection.extend(elements)
            elif isinstance(value, unicode):
                inst.eSet(feature, value.encode())
            elif not isinstance(value, str):
                inst.eSet(feature, feature.eType.from_string(value))
            else:
                inst.eSet(feature, value)
